/***************************************************************************
 *                                                                         *
 *   copyright (C) 2003 by Michael Buesch                                  *
 *   email: fsdeveloper@yahoo.de                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#include "filterpw.h"
#include <qobject.h>

#if 0
#include <iostream>
using std::cout;
using std::endl;
#endif

FilterPw::FilterPw()
{
  opt.enableFilter = false;
  chrset = 0;
}

FilterPw::FilterPw(const filterOpt *options, const charArray *charset)
{
  Q_ASSERT(options);
  opt = *options;
  chrset = charset;
}

FilterPw::~FilterPw()
{
}
/** set the filter-options (filter-rules) */
void FilterPw::setOpt(const filterOpt *options)
{
  Q_ASSERT(options);
  opt = *options;
}
/** set charset */
void FilterPw::setCharset(const charArray *charset)
{
  Q_ASSERT(charset);
  chrset = charset;
}

int FilterPw::scanNewChar(const string *tempPw, const char newChar)
{
  Q_ASSERT(chrset);
  if(!opt.enableFilter)
    return 0;
  if(!tempPw->length())
    return 0;

  if(opt.prevCharSeq > 1){
    if(!filter_CharSeq(tempPw, newChar))
      return 1;
  }
  if(opt.prevCharsetSeq > 1){
    if(!filter_CharsetSeq(tempPw, newChar))
      return 2;
  }
  return 0; // newChar ran through all filters and is ok.
}

bool FilterPw::filter_CharSeq(const string *tempPw, const char newChar)
{
//cout << "FILTER: prevCharSeq" << endl;

  const string::size_type pwlen = tempPw->length();
  if(!(pwlen + 1 < opt.prevCharSeq)){
    int i = pwlen - 1, equals = 0;
    for(; i > (int)(pwlen - opt.prevCharSeq); --i){
      if((*tempPw)[i] == newChar)
        ++equals;
      else
        break;
    }
    if((unsigned int)equals + 1 == opt.prevCharSeq)
      return false;
  }
  return true;
}

bool FilterPw::filter_CharsetSeq(const string *tempPw, const char newChar)
{
//cout << "FILTER: prevCharsetSeq" << endl;

  const string::size_type pwlen = tempPw->length();
  if(!(pwlen + 1 < opt.prevCharsetSeq)){
    int i = pwlen - 1, equals = 0;
    for(; i > (int)(pwlen - opt.prevCharsetSeq); --i){
      if(isInSameCs((*tempPw)[i], newChar))
        ++equals;
      else
        break;
    }
    if((unsigned int)equals + 1 == opt.prevCharsetSeq)
      return false;
  }
  return true;
}

/** find out the charset, the char belongs to. returns the members of the charset */
bool FilterPw::whichCs(const char ch, string *retcs)
{
  *retcs = chrset->ch_lower;
  if(loopThroughCs(retcs, ch))
    return true;
  *retcs = chrset->ch_upper;
  if(loopThroughCs(retcs, ch))
    return true;
  *retcs = chrset->ch_num;
  if(loopThroughCs(retcs, ch))
    return true;
  *retcs = chrset->ch_special;
  if(loopThroughCs(retcs, ch))
    return true;
  *retcs = chrset->ch_space;
  if(loopThroughCs(retcs, ch))
    return true;
  *retcs = chrset->ch_additional;
  if(loopThroughCs(retcs, ch))
    return true;
  return false;
}
/** used by whichCs() and isSameCs() to determine if ch is member of cs */
bool FilterPw::loopThroughCs(const string *cs, const char ch)
{
  int i = 0, cslen = cs->length();
  for(; i < cslen; ++i){
    if((*cs)[i] == ch)
      return true;
  }
  return false;
}
/** test if the two chars are in the same charset */
bool FilterPw::isInSameCs(const char ch1, const char ch2)
{
  string ch1_cs;
  if(!whichCs(ch1, &ch1_cs))
    return false;
  if(loopThroughCs(&ch1_cs, ch2))
    return true;
  return false;
}
