/***************************************************************************
 *                                                                         *
 *   copyright (C) 2003 by Michael Buesch                                  *
 *   email: fsdeveloper@yahoo.de                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#include "genpw.h"
#include "rndhelpwndimpl.h"
#include "pwgexception.h"

#include <klocale.h>
#include <kwin.h>
#include <kstandarddirs.h>

#include <qfileinfo.h>

#include <stdlib.h>

#include <iostream>
using namespace std;


GenPw::GenPw(KConfig *conf, QWidget* parentWidget) : QObject()
{
  confFile = conf;
  initMemb(parentWidget);
}
GenPw::GenPw(KConfig *conf, pwRules *rules, const filterOpt *filterOptions, QWidget* parentWidget) : QObject()
{
  confFile = conf;
  initMemb(parentWidget);
  curRules = *rules;
  setFilterOpt(filterOptions);
}
GenPw::~GenPw()
{
  delete randomizer;
}
/** initialize character-arrays */
void GenPw::initChAr()
{
  chAr.ch_lower   = "abcdefghijklmnopqrstuvwxyz";
  chAr.ch_upper   = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
  chAr.ch_num     = "0123456789";
  chAr.ch_special = "!\"$%&/()=?,.-;:_+";
  chAr.ch_space   = " ";
}
/** init class members */
void GenPw::initMemb(QWidget* parentWidget)
{
  initChAr();
  // initialize randomizer bg-picture
  QString bgpic(confFile->readEntry("rndBackPix"));
  QFileInfo fi(bgpic);
  if(!fi.isReadable()) // no pic configured. Fall back to default
    bgpic = ::locate("data", PWG_DEFAULT_BG_PIC);
  randomizer = new GenRand(bgpic, UNSIGNED_LONG_MAX, parentWidget);
  randomizer->hide();
  connect(randomizer, SIGNAL(newRand_signal(unsigned long)), this, SLOT(newRndAvailable_slot(unsigned long)));
  connect(randomizer, SIGNAL(closedByUser_signal()), this, SIGNAL(closedByUser_signal()));
}
/** set the rules from which the password will be generated */
void GenPw::setRules(pwRules *rules)
{
  curRules = *rules;
}
/** generates a new password and returns it (*pw) */
void GenPw::genNow()
{
  // read and select charset
  curCharset = "";
  tmpPw = "";
  if(curRules.useLower)
    curCharset.append(chAr.ch_lower);
  if(curRules.useUpper)
    curCharset.append(chAr.ch_upper);
  if(curRules.useNum)
    curCharset.append(chAr.ch_num);
  if(curRules.useSpecial)
    curCharset.append(chAr.ch_special);
  if(curRules.useSpace)
    curCharset.append(chAr.ch_space);
  if(curRules.additChars.length()){
    curCharset.append(curRules.additChars);
    chAr.ch_additional = curRules.additChars;
  } else
    chAr.ch_additional = "";

  // (pseudo)randomize the charset
  string tmp;
  int pos;
  do{
    pos = randomizer->srnd.genRnd(curCharset.length());
    tmp.append(curCharset, pos, 1);
    curCharset.erase(pos, 1);
  } while(curCharset.length());
  curCharset = tmp;

  // transmit charset to filter
  filter.setCharset(&chAr);
  // initialize "graphical randomizer"
  randomizer->setMaxRnd(curCharset.length());

  // show help-dialog is configured to do so
  confFile->setGroup("GLOBAL");
  if(confFile->readBoolEntry("showRandomizerHelp", true)){
    RndHelpWndImpl *helpWnd = new RndHelpWndImpl(confFile, randomizer);
    helpWnd->show();
    KWin::setActiveWindow(helpWnd->winId()); /* FIXME: Doesn't work, because rnd-window is displayed
                                              * just after bringing this to FG. But I think brining to FG
					      * is not needed either.
					      */
  }
  // start the "graphical randomizer"
  randomizer->start();
}
/** new random number is available from "randomizer" */
void GenPw::newRndAvailable_slot(unsigned long rnd)
{
#if 0
/* BEGIN: Filter testsuite */
  if(!filter.scanNewChar(&tmpPw, 'k'))
  tmpPw += "k";
/* END: Filter testsuite */
#endif

  char newChar = curCharset[rnd];
  if(!filter.scanNewChar(&tmpPw, newChar)){
    tmpPw += newChar;
    int curlen = tmpPw.length();
    emit pushProgressBar_signal(curRules.length, curlen);
    if((unsigned int)curlen == curRules.length){
      // password calculated completely
      randomizer->stop();
      emit newPw_signal(tmpPw);
    }
  }
  else
    emit rejByFilter_signal(newChar);
}
