/***************************************************************************
 *                                                                         *
 *   copyright (C) 2003 by Michael Buesch                                  *
 *   email: fsdeveloper@yahoo.de                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#ifndef GENPW_H
#define GENPW_H

#include <qobject.h>

#include <kconfig.h>

#include "genrand.h"
#include "filterpw.h"

#define PWG_DEFAULT_BG_PIC	"pwg/pics/pwg-back2.png"

/** generate pw using this rules */
typedef struct {
  unsigned int length;
  bool useLower;
  bool useUpper;
  bool useNum;
  bool useSpecial;
  bool useSpace;
  string additChars;
} pwRules;

/**this class generates a password
  *@author Michael Buesch
  */

class GenPw : public QObject{
  Q_OBJECT
public: 
  GenPw(KConfig *conf, QWidget* parentWidget = 0);
  GenPw(KConfig *conf, pwRules *rules, const filterOpt *filterOptions, QWidget* parentWidget = 0);
  ~GenPw();
  /** set the rules from which the password will be generated */
  void setRules(pwRules *rules);
  /** set filter options */
  void setFilterOpt(const filterOpt *filterOptions) { filter.setOpt(filterOptions); }
  /** generates a new password and returns it (*pw) */
  void genNow();
  /** changes the background of the randomizer */
  void randChangeBg(QString pixPath) { randomizer->setBackgr(pixPath); }

protected:
  /** current rules to be used */
  pwRules curRules;
  /** calculates random numbers */
  GenRand *randomizer;
  /** current charset (in current calculation) */
  string curCharset;
  /** tempoary password */
  string tmpPw;
  /** for password-filtering */
  FilterPw filter;

protected:
  /** init class members */
  void initMemb(QWidget* parentWidget = 0);

protected:
  charArray chAr;
  /** config file */
  KConfig *confFile;
  /** initialize character-arrays */
  void initChAr();
signals: // Signals
  /** new password available */
  void newPw_signal(string pw);
  /** set progress-bar (that is in main-wnd) to position.
      'totalSteps' is the number of total progress-steps
      'progress' is the current progress position */
  void pushProgressBar_signal(int totalSteps, int progress);
  /** randomizer closed by user (canceled) */
  void closedByUser_signal();
  /** some filter didn't let it through */
  void rejByFilter_signal(char rej);

public slots: // Public slots
  /** new random number is available from "randomizer" */
  void newRndAvailable_slot(unsigned long rnd);
};

#endif
