/***************************************************************************
 *                                                                         *
 *   copyright (C) 2003 by Michael Buesch                                  *
 *   email: fsdeveloper@yahoo.de                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#include "genrand.h"

#include <qlabel.h>
#include <qfileinfo.h>
#include <qimage.h>


GenRand::GenRand(QString backgrPix, unsigned long maxRnd, QWidget* parent, const char *name)
 : GenWnd(parent, name)
{
  Q_ASSERT(maxRnd != 0);
  max = maxRnd;

  // init UI
  setMouseTracking(true);
  setFixedSize(QSize(200, 200));
  setBackgr(backgrPix);
}

GenRand::~GenRand()
{
}

void GenRand::mouseMoveEvent(QMouseEvent *e)
{
  static unsigned int skipped = 0; // already skipped moves
  static bool nextUseX = (srnd.genRnd(2) ? true : false);     // next time use X or Y position of mouse-cursor
  static unsigned int nextSkip = srnd.genRnd(MAX_SKIP); // next time skip this number of moves

  if(e->y() > 200 || e->y() < 0 || e->x() > 200 || e->x() < 0)
    return;

  if(skipped < nextSkip)
    ++skipped;
  else{
    // generate random number
    unsigned long rndVal = ((nextUseX ? e->x() : e->y()) * (srnd.genRnd(21474836))) % max;
    emit newRand_signal(rndVal); // emit new random number using signal
    nextSkip = srnd.genRnd(MAX_SKIP);
    skipped = 0;
    nextUseX = (srnd.genRnd(2) ? true : false);
  }
}

void GenRand::setMaxRnd(unsigned long maxRnd)
{
  Q_ASSERT(maxRnd != 0);
  max = maxRnd;
}

void GenRand::start()
{
  show();
}

void GenRand::stop()
{
  hide();
}

void GenRand::closeEvent(QCloseEvent *e)
{
  /* window closed by the user!
   * now inform the main-window of this.
   */
  emit closedByUser_signal();
  e->accept();
}

bool GenRand::setBackgr(QString pixPath)
{
  if(pixPath.isEmpty())
    return false;
  QFileInfo fi(pixPath);
  if(!fi.isFile() || !fi.isReadable())
    return false;
  backp.load(pixPath);
  if(backp.width() != 200 || backp.height() != 200)
    backp.convertFromImage(backp.convertToImage().scale(200, 200));
  background->setPixmap(backp);
  return true;
}

