/***************************************************************************
 *                                                                         *
 *   copyright (C) 2003 by Michael Buesch                                  *
 *   email: fsdeveloper@yahoo.de                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#include "outputwndimpl.h"
#include "pwm_stub.h"

#include <qclipboard.h>
#include <qapplication.h>

#include <kprocess.h>
#include <kmessagebox.h>
#include <klocale.h>

#include <unistd.h>


OutputWndImpl::OutputWndImpl(QWidget* parent, const char *name)
 : OutputWnd(parent, name)
{
}

OutputWndImpl::~OutputWndImpl()
{
}

void OutputWndImpl::closeEvent(QCloseEvent *e)
{
  e->accept();
  delete this;
}

void OutputWndImpl::copyClipButton_slot()
{
  QClipboard *cb = QApplication::clipboard();
  cb->setText(outputLineEdit->text());
}

void OutputWndImpl::copyPwMButton_slot()
{
  KProcess pwmproc;
  pwmproc << "pwm";
  if(!pwmproc.isRunning()){
    if(!pwmproc.start(KProcess::DontCare)){
      KMessageBox::error(this, i18n("Couldn't start the PassWordManager \"PwM\"!\n"
         "Please check if you have installed PwM correctly.\n"
	 "PwM is available from this webpage: http://www.8ung.at/tuxsoft"),
	 i18n("PwM failed."));
      return;
    }
  }

  PwM_stub pwmstub("pwm", "pwm");
  struct timespec haltTime;
  haltTime.tv_sec = 0;
  haltTime.tv_nsec = 300;
  while(pwmstub.dcop_isRunning() != true)
    nanosleep(&haltTime, 0);
  pwmstub.dcop_addPwd(outputLineEdit->text());
  if(!pwmstub.ok()){
    KMessageBox::error(this, i18n("Failed to submit password to PwM!\n"
       "DCOP communication error."),
       i18n("DCOP error."));
    return;
  }
}

