/***************************************************************************
 *                                                                         *
 *   copyright (C) 2003 by Michael Buesch                                  *
 *   email: fsdeveloper@yahoo.de                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#include "simplernd.h"

#include <stdlib.h>
#include <time.h>

#include <iostream>

using std::cerr;
using std::endl;


SimpleRnd::SimpleRnd()
{
  // probe for random devices
  randomDevice = fopen("/dev/urandom", "r");
  if(randomDevice){
    useDev = true;
    return;
  }
  randomDevice = fopen("/dev/random", "r");
  if(randomDevice){
    useDev = true;
    return;
  }

  // fall back to rand()
  randomDevice = 0;
  srand(time(0));
  useDev = false;
}

SimpleRnd::~SimpleRnd()
{
  if(useDev)
    if(fclose(randomDevice))
      cerr << "ERROR: failed closing the random-device!" << endl;
}

unsigned long SimpleRnd::rndDev(unsigned long maxRnd)
{
  return ((getc(randomDevice) *
    getc(randomDevice) *
    getc(randomDevice) *
    getc(randomDevice)) % maxRnd);
}

unsigned long SimpleRnd::rndRand(unsigned long maxRnd)
{
  return rand() % maxRnd;
  /* Note: rand() does only return int! */
  /* In the GNU library, it is 2147483647, which is the largest signed integer
   * representable in 32 bits. In other libraries, it may be as low as 32767.
   */
}

unsigned long SimpleRnd::genRnd(unsigned long maxRnd)
{
  if(useDev)
    return rndDev(maxRnd);
  return rndRand(maxRnd);
}
